@@ -11,11 +11,15 @@ use File::Spec;
 use Sphinx::Config;
 use Errno qw/ECHILD/;
 
+our $VERSION = '0.04';
+
 __PACKAGE__->mk_accessors(qw/config_file 
 			  pid_file 
 			  bindir 
 			  searchd_args 
+                          searchd_sudo
 			  indexer_args 
+                          indexer_sudo
 			  process_timeout 
 			  debug/);
 
@@ -137,20 +141,20 @@ sub _wait_for_proc {
 
 sub _system_with_status
 {
-    my ($command) = @_;
+    my (@args) = @_;
 
     local $SIG{CHLD} = 'IGNORE';
-    my $status = system($command);
+    my $status = system(@args);
     unless ($status == 0) {
         if ($? == -1) {
 	    return '' if $! == ECHILD;
-            return "$command failed to execute: $!";
+            return "@args failed to execute: $!";
         }
         if ($? & 127) {
-            return sprintf("$command died with signal %d, %s coredump\n",
+            return sprintf("@args died with signal %d, %s coredump\n",
                            ($? & 127),  ($? & 128) ? 'with' : 'without');
         }
-        return sprintf("$command exited with value %d\n", $? >> 8);
+        return sprintf("@args exited with value %d\n", $? >> 8);
     }
     return '';
 }
@@ -182,6 +186,7 @@ sub get_searchd_pid {
 
 sub start_searchd {
     my $self = shift;
+    my $ok_if_running = shift;
 
     my $pidfile = $self->_find_pidfile;
     warn "start_searchd: Checking pidfile $pidfile" if $self->debug;
@@ -189,21 +194,31 @@ sub start_searchd {
     if ( -f "$pidfile" ) {
 	my $pid = Path::Class::file($pidfile)->slurp(chomp => 1);
 	warn "start_searchd: Found PID $pid" if $self->debug;
-	die "searchd is already running" if $pid && @{$self->_findproc($pid, qr/searchd/)};
+	if ($pid && @{$self->_findproc($pid, qr/searchd/)}) {
+	    return if $ok_if_running;
+	    die "searchd is already running, PID $pid";
+	}
     }
 
-    my $searchd = $self->_find_exe('searchd');
-    my @args = ("--config", $self->{_config_file}->stringify);
+    my @args;
+    if (my $sudo = $self->searchd_sudo) {
+	if (ref($sudo) ne 'ARRAY') {
+	    $sudo = [ split(/\s+/, $sudo) ];
+	}
+	push(@args, @$sudo);
+    }
+    push(@args, $self->_find_exe('searchd'));
+    push(@args, "--config", $self->{_config_file}->stringify);
     push(@args, @{$self->searchd_args}) if $self->searchd_args;
-    warn("Executing $searchd " . join(" ", @args)) if $self->debug;
+    warn("Executing " . join(" ", @args)) if $self->debug;
 
     local $SIG{CHLD} = 'IGNORE';
     my $pid = fork();
     die "Fork failed: $!" unless defined $pid;
     if ($pid == 0) {
 	fork() and exit; # double fork to ensure detach
-	exec("$searchd", @args)
-	    or die("Failed to exec $searchd}: $!");
+	exec(@args)
+	    or die("Failed to exec @args}: $!");
     }
 
     die "Searchd not running after timeout" 
@@ -211,14 +226,32 @@ sub start_searchd {
 
 }
 
+sub _kill {
+    my ($self, $sig, @pids) = @_;
+
+    if (my $sudo = $self->searchd_sudo) {
+	my @args;
+	if (ref($sudo) ne 'ARRAY') {
+	    $sudo = [ split(/\s+/, $sudo) ];
+	}
+	push(@args, @$sudo);
+	push(@args, 'kill');
+	push(@args, "-$sig", @pids);
+	_system_with_status(@args);
+    }
+    else {
+	kill $sig, @pids;
+    }
+}
+
 sub stop_searchd {
     my $self = shift;
 
     my $pids = $self->get_searchd_pid;
     if (@$pids) {
-	kill 15, @$pids;
+	$self->_kill(15, @$pids);
 	unless ($self->_wait_for_death(@$pids)) {
-	    kill 9, @$pids;
+	    $self->_kill(9, @$pids);
 	    unless ($self->_wait_for_death(@$pids)) {
 		die "Failed to stop searchd PID " . join(", ", @$pids) . ", even with sure kill";
 	    }
@@ -230,7 +263,7 @@ sub stop_searchd {
 sub restart_searchd {
     my $self = shift;
     $self->stop_searchd;
-    $self->start_searchd;
+    $self->start_searchd(1);
 }
 
 sub reload_searchd {
@@ -239,7 +272,7 @@ sub reload_searchd {
     my $pids = $self->get_searchd_pid;
     if (@$pids) {
 	# send HUP
-	kill 1, @$pids;
+	$self->_kill(1, @$pids);
     }
     else {
 	$self->start_searchd;
@@ -250,32 +283,36 @@ sub run_indexer {
     my $self = shift;
     my @extra_args = @_;
 
-    my $indexer = $self->_find_exe('indexer');
-    warn("Using indexer $indexer") if $self->debug;
+    my @args;
+    if (my $sudo = $self->indexer_sudo) {
+	if (ref($sudo) ne 'ARRAY') {
+	    $sudo = [ split(/\s+/, $sudo) ];
+	}
+	push(@args, @$sudo);
+    }
+    push(@args, my $indexer = $self->_find_exe('indexer'));
+
+    warn("Using indexer @args") if $self->debug;
     die "Cannot execute Sphinx indexer binary $indexer" unless -x "$indexer";
 
     my $config = $self->config_file || $default_config_file;
-    my $cmd = "$indexer --config $config";
-    $cmd .= ' ' . join(" ", @{$self->indexer_args}) if $self->indexer_args;
-    $cmd .= ' ' . join(" ", @extra_args) if @extra_args;
+    push(@args, "--config", $config);
+    push(@args, @{$self->indexer_args}) if $self->indexer_args;
+    push(@args, @extra_args) if @extra_args;
 
-    if (my $status = _system_with_status($cmd)) {
+    if (my $status = _system_with_status(@args)) {
 	die $status;
     }
 }
 
+1;
+
+__END__ 
+
 =head1 NAME
 
 Sphinx::Manager - Sphinx search engine management (start/stop)
 
-=head1 VERSION
-
-Version 0.02
-
-=cut
-
-our $VERSION = '0.02';
-
 =head1 SYNOPSIS
 
     use Sphinx::Manager;
@@ -345,6 +382,10 @@ Enable/disable debugging messages, or read back debug status.
 
 Set/get the time (in seconds) to wait for processes to start or stop.
 
+=head2 searchd_bin
+
+Optional full path to searchd.  If not specified, normal rules for finding an executable in the PATH environment are used.
+
 =head2 searchd_args
 
     $mgr->searchd_args(\@args)
@@ -355,13 +396,41 @@ start_searchd.  These should be in the form of an array, each entry comprising
 one option or option argument.  Arguments should exclude '--config CONFIG_FILE',
 which is included on the command line by default.
 
+=head2 searchd_sudo
+
+Optional sudo wrapper for running searchd as root or another user.
+Set to 'sudo', or 'sudo -u USER', or sudo with any other options.  If
+not set, searchd is run as the current user.  Of course, for this to
+work, sudo privileges should be enabled for the current user and
+typically without a password.  Could also be set to the name of any
+other wrapper program to provide custom functionality.
+
+=head2 indexer_args
+
+    $mgr->indexer_args(\@args)
+    $args = $mgr->indexer_args;
+
+Set/get the extra command line arguments to pass to the indexer program when
+started using run_indexer.  These should be in the form of an array, each entry
+comprising one option or option argument.  Arguments should exclude '--config
+CONFIG_FILE', which is included on the command line by default.
+
+
+=head2 indexer_sudo
+
+Same as L<searchd_sudo> above, for running the indexer.
+
 =head1 METHODS
 
 =head2 start_searchd
 
     $mgr->start_searchd;
+    $mgr->start_searchd($ok_if_running);
 
-Starts the Sphinx searchd daemon.  Dies if searchd cannot be started or if it is already running.
+Starts the Sphinx searchd daemon.  Unless $ok_if_running is set, dies if searchd
+cannot be started or if it is already running.  If $ok_if_running is set and
+searchd is found to be already running, returns quietly without trying to start
+it.
 
 =head2 stop_searchd
 
@@ -374,7 +443,7 @@ Stops Sphinx searchd daemon.  Dies if searchd cannot be stopped.
 
     $mgr->restart_searchd;
 
-Stops and thens starts the searchd daemon.
+Stops and then starts the searchd daemon.
 
 =head2 reload_searchd
 
@@ -394,16 +463,6 @@ checked for other searchd processes running with the specified config file; if
 found, all are returned in the array.
 
 
-=head2 indexer_args
-
-    $mgr->indexer_args(\@args)
-    $args = $mgr->indexer_args;
-
-Set/get the extra command line arguments to pass to the indexer program when
-started using run_indexer.  These should be in the form of an array, each entry
-comprising one option or option argument.  Arguments should exclude '--config
-CONFIG_FILE', which is included on the command line by default.
-
 =head2 run_indexer(@args)
 
 Runs the indexer program; dies on error.  Arguments passed to the indexer are
@@ -470,4 +529,3 @@ under the same terms as Perl itself.
 
 =cut
 
-1; # End of Sphinx::Manager
